/* Arduino Range finder with HC-SR04 or Parallax Ping))) tm Sensor
  -- Changelog --
  2017/12/30 code version V1.01
  -Added a timeout to turn off the display when standstill at STOP distance

  2016/11/12 code version V1.00 open source by AppsByDavideV
  -First release
  --------------

  >Web:       http://appsbydavidev.duckdns.org
  >Facebook:  www.facebook.com/appsbydavidev.devCounter
  >Twitter:   https://twitter.com/appsbydavidev
  >Instagram: https://instagram.com/appsbydavidev/
  >Google+:   https://plus.google.com/112943455419093026838/posts
  >Youtube:   https://www.youtube.com/channel/UCNY3KkW72IEFlLiBgTBdoKw
  >Email:     vdavidex@gmail.com
  @appsbydavidev #appsbydavidev #arduinorangefinder


  This sketch reads a HC-SR04/Ping))) ultrasonic rangefinder and returns the
  distance to the closest object in range. To do this, it sends a pulse
  to the sensor to initiate a reading, then listens for a pulse
  to return.  The length of the returning pulse is proportional to
  the distance of the object from the sensor.

  How it works in this sketch:
  -The sensor checks range every pingDelay (ms)
  -The returned distance is processed with a digital low pass filter to keep it more stable
  -The result is compared with some constants (see #define below)
  -if the result is > OFF_DISTANCE the display is kept off (no messages)
  -If the result is in the range of FAR_DISTANCE to OFF_DISTANCE the message is Far..
  -If the result is below FAR_DISTANCE the distance is shown
  -If the result is below STOP_DISTANCE the message is STOP

*/

#include "Arduino.h"
#include <fourMatrix.h>     //display library

#define SENSOR_PING 1       //uncomment if you are using an Parallax PING))) tm range finder
//#define SENSOR_HCSR04 1     //uncomment if you are using an HC-SR04 range finder
#define DISTANCE_CM 1       //uncomment for distance in Inches

//Sensor pins for HC-SR04 or Ping)))
const int triggerPin = 6;       // Trigger pin, only for HC-SR04 sensor
const int echoPin = 7;          // Echo pin on HC-SR04 or SIG pin on Ping)))

//Measurement
const int OFF_DISTANCE = 120;   // Distance in cm for no messages (display off)
const int FAR_DISTANCE = 70;    // Distance in cm for FAR message on display
const int STOP_DISTANCE = 15;   // Distance in cm for STOP message on display

unsigned int pingDelay = 100;   // How frequently are we going to send out a ping (in milliseconds).
unsigned long nextPing;         // Holds the next ping time.
unsigned long displayOff;       // Holds the timeout to turn off the display when at Stop position
const unsigned long DISPLAY_OFF = 10000; //time in mSec after that the display will be off

//Matrix display pin assignment
const int latchPin = 9;
const int clockPin = 8;
const int dataPin = 10;
const int checkPin = 13;        //arduino led on pin 13

unsigned int lastDistance = 0;
unsigned int cm = 0;            //raw centimeters
unsigned int LPcm = 0;          //centimeters after low pass filter
float lp = 0.4;                 //low pass smooth
boolean stopFlag = false;

fourMatrix m(dataPin, latchPin, clockPin, 4); // (datapin,latchpinn, clockpin,number of modules)

void setup() {
#if defined(SENSOR_PING)
  //setup for Parallax Ping))) tm
  pinMode(echoPin, OUTPUT);
  digitalWrite(echoPin, LOW);
#endif

#if defined(SENSOR_HCSR04)
  pinMode(triggerPin, OUTPUT);
  digitalWrite(triggerPin, LOW);
  pinMode(echoPin, INPUT);
#endif

  Serial.begin(57600);
  m.init(); //initialize display
  m.printMsgToMatrix(" Wait ", 0);

  nextPing = millis() + pingDelay;           // schedule next ping
}

void loop() {
  if (millis() >  nextPing) {      // time expired, make measurement
    cm = pingCm(5);
    LPcm = LowPass(cm, lp, LPcm);             //smoothed value
    nextPing = millis() + pingDelay;          // schedule next ping  with last delay in case LPcm doesn't change value
    //Serial.println("Average: " + String(LPcm) + " ,Real: " + String(cm) +" cm");    //debug only, comment after test

    if (LPcm != lastDistance) {
      lastDistance = LPcm;                    //save new value

      if (LPcm == 0) {
        //Reading error, check sensor connections
        m.clearMatrix();
        m.printMsgToMatrix("Error ", 0);
        stopFlag = false;
        pingDelay = 5000;
        nextPing = millis() + pingDelay;           // schedule next ping

      } else if (LPcm > FAR_DISTANCE && LPcm < OFF_DISTANCE) {
        //Far distance, show the 'FAR...' message if distance is between FAR & OFF
        m.printMsgToMatrix("Far... ", 0);
        stopFlag = false;
        pingDelay = 250;
        nextPing = millis() + pingDelay;           // schedule next ping

      } else if (LPcm < STOP_DISTANCE) {
        //Stop distance, show a fixed stop message
        if (stopFlag == false) {
          m.clearMatrix();
          m.printMsgToMatrix("Stop!!! ", 0);
          stopFlag = true;
          pingDelay = 100;
          displayOff = millis() + DISPLAY_OFF;
          nextPing = millis() + pingDelay;           // schedule next ping
        } 

      } else if (LPcm >= OFF_DISTANCE) {
        //Off distance, turn off display & slow down ping interval
        m.clearMatrix();
        stopFlag = false;
        pingDelay = 1000;
        nextPing = millis() + pingDelay;           // schedule next ping

      } else {
        //Distance in range of measurement, show distance on display & speed up ping interval
        String Blank = "    "; //4x spaces
        Blank.remove(0, String(LPcm).length()); //adjust message length
        m.printMsgToMatrix(Blank + String(LPcm) + "cm ", 0);
        stopFlag = false;
        pingDelay = 100;
        nextPing = millis() + pingDelay;           // schedule next ping
      }
    }
  }
  // Do other stuff here not related to ping
 
  //check if it's time to turn off the display
  if ((millis() > displayOff) && (stopFlag==true)) {
      m.clearMatrix();
   }
        
}


//Routines

//low pass digital filter
float LowPass(float Value, float filterValue, float LastValue) {
  if (filterValue > 1) {
    filterValue = 0.99;
  }
  if (filterValue <= 0)  {
    filterValue = 0;
  }
  return (Value * (1 - filterValue)) + (LastValue * filterValue);
}

//return distance in centimeters
long pingCm(int Reads) {
  //start meausement cycle by 10microsec pulse on trigger pin
  long Buffer[Reads];
  long average = 0;
  for (int i = 0; i < Reads; i++) {
#if defined(SENSOR_HCSR04)
    digitalWrite(triggerPin, HIGH);
    delayMicroseconds(10);
    digitalWrite(triggerPin, LOW);
#endif
#if defined(SENSOR_PING)
    pinMode(echoPin, OUTPUT);
    digitalWrite(echoPin, HIGH);
    delayMicroseconds(5);
    digitalWrite(echoPin, LOW);
    pinMode(echoPin, INPUT);
#endif
    Buffer[i] = pulseIn(echoPin, HIGH);
    //Serial.print(String(Buffer[i]) + "\t");  //debug only
    delay(25);    //do not remove
  }
  for (int i = 0; i < Reads; i++) {
    average = average + Buffer[i];
  }
  return ms2cm(average / Reads);
}

//return distance in Inches
long pingIn(int Reads) {
  //start meausement cycle by 10microsec pulse on trigger pin
  long Buffer[Reads];
  long average = 0;
  for (int i = 0; i < Reads; i++) {
#if defined(SENSOR_HCSR04)
    digitalWrite(triggerPin, HIGH);
    delayMicroseconds(10);
    digitalWrite(triggerPin, LOW);
#endif
#if defined(SENSOR_PING)
    pinMode(echoPin, OUTPUT);
    digitalWrite(echoPin, HIGH);
    delayMicroseconds(5);
    digitalWrite(echoPin, LOW);
    pinMode(echoPin, INPUT);
#endif
    Buffer[i] = pulseIn(echoPin, HIGH);
    //Serial.print(String(Buffer[i]) + "\t");  //debug only
    delay(25);  //do not remove
  }
  for (int i = 0; i < Reads; i++) {
    average = average + Buffer[i];
  }
  return ms2In(average / Reads);
}

//return Inches from uS
long ms2In(long microseconds) {
  return microseconds / 148;
}

//return centimeters from uS
long ms2cm(long microseconds) {
  return microseconds / 58;
}


